% -------------------------------------------------------------------------
% License GPL v3.0
% Author: Borja Ayerdi <borja.ayerdi@ehu.es>
% Computational Intelligence Group <www.ehu.es/ccwintco>
% University Of The Basque Country (UPV/EHU) 
% Use this at your own risk!
% -------------------------------------------------------------------------

function [herf inforotar numClass] = trainHERF(X,Y, dim)

numClass=max(Y);
NF = size(X,2);


for i=1:dim
    
    % Given:
    % X: the objects in the training data set (an N x n matrix)
    % Y: the labels of the training set (an N x 1 matrix)
    % K: the number of subsets
    % NF: the number of total features
    % {w1,w2,.., wc}: the set of class labels
    %L

    % Prepare the rotaton matrix R:
    % Split F (the feature set) into K subsets Fij (for j=1,..,K)
    % K is a random value between 1 and NF.
    % We want at least 1 feature for each subset.
    K= 1 + round((NF-1)*rand);

    FK = zeros(K,NF);
    for f = 1:K
        numSelecFeatures = 1 + round((NF-1)*rand);
        rp = randperm(NF);
        FK(f,rp(1:numSelecFeatures)) = 1;
    end
    
    R = zeros(NF);
    for j = 1:K
        % Let Xij be the data set X for the features in Fij
        pos=find(FK(j,:)==1);
        XXij = X(:,pos);
        % Eliminate from Xij a random subset of classes
        
        %%% DIMENSIONALITY REDUCTION %%%
        % Apply quartimax on X'ij to obtain the coefficients in a matrix Cij
        %[L1,COEFF] = rotatefactors(XXij,'method','quartimax');
        
        % Apply PCA
        [COEFF,SCORE] = princomp(XXij);

        % Arrange the Cij for j=1,..K in a rotation matriz Ri
        % Construct Ri by rearrangin the columns of Ri so as to match the order
        % of features in F.
        for indI = 1:length(COEFF)
            for indJ = 1:length(COEFF)
                R(pos(indI), pos(indJ)) = COEFF(indI,indJ);
            end
        end        
    end

    Xrot = X*R-repmat(mean(X),[size(X,1) 1]);
    
    inforotar{i}.R=R;
    inforotar{i}.mediaX=mean(X);
    
    if (mod(i,2)==0)
        elm = trainELM(Xrot,Y);
        herf{i}=elm;
    elseif (mod(i,2)==1)
        tree=classregtree(Xrot,Y);
        herf{i}=tree;
    end
    
end


end